package com.altinity.opsview.odw.dao;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.HostGroup;
import com.altinity.opsview.odw.model.ReportConfig;
import com.altinity.opsview.odw.reports.ReportCriteria;

public class ODWReportConfigDAO extends ODWDAO {
	
	/**
	 * This method returns a map of host groups for a 
	 * specific report - complete with warning and critical
	 * thresholds - keyed by their host group name
	 */
	public Map <String, HostGroup> getHostGroupsForReport(String reportName) {
		
		// First get the host groups by name
		Map <String, HostGroup> allHostGroups =
			getHostGroupsByName();
		
		// Get the default host group - we will need this
		// for any host groups that do not have thresholds
		// specified
		HostGroup defaultHG = allHostGroups.get("");
		if ( defaultHG == null )
			throw new RuntimeException("No default host group found");	
		
		List <String> hostGroupNames = getHostGroupNamesForReport(reportName);
		
		// Create linked hash map to retain order of host groups
		Map <String, HostGroup> hostGroupsForReport = new LinkedHashMap<String, HostGroup>();
		
		// Now update host groups thresholds - that is, update
		// host groups that have no thresholds specified with
		// the default thresholds
		for ( String hostGroupName : hostGroupNames ) {
			
			HostGroup hostGroup = allHostGroups.get(hostGroupName);
			
			// First check if host group is null
			// If so, create new host group with default values
			if ( hostGroup == null ) {
				
				hostGroup = new HostGroup(hostGroupName);			
				hostGroup.setCritical(defaultHG.getCritical());
				hostGroup.setWarning(defaultHG.getWarning());
			}
			
			// Check if warning level should be inherited 
			// from default values
			if ( hostGroup.getWarning() == 0.0d )
				hostGroup.setWarning(defaultHG.getWarning());
			
			// Check if critical level should be inherited
			// from default values
			if ( hostGroup.getCritical() == 0.0d )
				hostGroup.setCritical(defaultHG.getCritical());
			
			hostGroupsForReport.put(hostGroupName, hostGroup);
		}
		
		return hostGroupsForReport;
	}
	
	/**
	 * This method returns a list of host group names
	 * for the specifed report
	 */
	private List<String> getHostGroupNamesForReport(String reportName) {
		
		// Now create the query the host group names for this report	
		StringBuffer query = new StringBuffer();
		query.append("SELECT reports.selectionhostgroups.hostgroup ");
		query.append("FROM reports.reports, reports.selectionhostgroups ");
		query.append("WHERE reports.reports.name = :reportName ");
		query.append("AND reports.reports.selection = reports.selectionhostgroups.selection ");
		query.append("ORDER BY reports.selectionhostgroups.hostgroup");

		// Execute the query for the host group names for this report
		MapSqlParameterSource namedParameters = new MapSqlParameterSource("reportName", reportName);
		List <String> hostGroupNames = 
			getJdbcTemplate().query(query.toString(), new SimpleStringMapper(), namedParameters);
		
		return hostGroupNames;
	}
	
	
	/**
	 * This method returns a map of all host groups - with 
	 * no threshold data - keyed by their host group name
	 */
	private Map<String, HostGroup> getHostGroupsByName() {
		
		// First get thresholds for host groups
		StringBuffer query = new StringBuffer();
		query.append("SELECT * ");
		query.append("FROM reports.generic_availability_targets_hostgroup ");
		List <HostGroup> hostGroups = getJdbcTemplate().query(query.toString(), new HostGroupMapper());
		
		// Generate map of host groups
		Map <String, HostGroup> hostGroupsByName = new HashMap<String, HostGroup>();
		for ( HostGroup hostGroup : hostGroups ) {
			hostGroupsByName.put(hostGroup.getName(), hostGroup);
		}
		
		return hostGroupsByName;
	}
	
	/**
	 * 
	 */
	private class HostGroupMapper implements ParameterizedRowMapper<HostGroup> {

		public HostGroup mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			HostGroup hg = new HostGroup(rs.getString(2));
			hg.setWarning(new Double(rs.getDouble(3)));
			hg.setCritical(new Double(rs.getDouble(4)));

			return hg;
		}
	}
	
	/**
	 *
	 */
	public ReportConfig getReportConfigByReportId(String reportId) throws ReportException {
		
		StringBuffer query = new StringBuffer();
		query.append("SELECT reports.id, ");
		query.append("reports.name, ");
		query.append("selections.name selection, ");
		query.append("reports.format, ");
		query.append("report_types.name description, ");
		query.append("reports.type typeId, ");
		query.append("weekly_availability_keyword_critical_states.label, ");
		query.append("weekly_availability_keyword_critical_states.start_of_week ");
		query.append("FROM (report_types, ");
		query.append("reports LEFT JOIN selections ON selections.id = reports.selection) ");
		query.append("LEFT JOIN weekly_availability_keyword_critical_states ON reports.id = weekly_availability_keyword_critical_states.reportid ");
		query.append("WHERE reports.id = :reportId ");
		query.append("and report_types.id=reports.type");

		ParameterizedRowMapper<ReportConfig> mapper = new ParameterizedRowMapper<ReportConfig>() {
			public ReportConfig mapRow(ResultSet rs, int rowNum) throws SQLException {
				ReportConfig reportConfig = new ReportConfig();
				reportConfig.setId(rs.getInt("id"));
				reportConfig.setName(rs.getString("name"));
				reportConfig.setSelection(rs.getString("selection"));
				reportConfig.setFormat(rs.getString("format"));
				reportConfig.setType(rs.getString("description"));
				reportConfig.setTypeId(rs.getInt("typeId"));
				reportConfig.setLabel(rs.getString("label"));
				reportConfig.setStartOfWeek(rs.getInt("start_of_week"));
				return reportConfig;
			}
		};

		MapSqlParameterSource namedParameters = new MapSqlParameterSource("reportId", reportId);
		
		// Throw runtime exception if report id is invalid
		ReportConfig reportConfig = null;
		try {
			reportConfig = getJdbcTemplate().queryForObject(query.toString(), mapper, namedParameters);
		}
		catch(EmptyResultDataAccessException e) {
			throw new ReportException("Invalid report ID: '" + reportId + "'");
		}
		
		return reportConfig;
	}

	/**
	 * 
	 */
	public List <String> getReportSections(ReportCriteria rc) {
		
		StringBuffer query = new StringBuffer();
		query.append("SELECT hostgroup ");
		query.append("FROM reports.reports, reports.selectionhostgroups ");
		query.append("WHERE reports.reports.selection = reports.selectionhostgroups.selection ");
		query.append("and reports.reports.id = :reportId");
		
		// Execute the query and return a list of availability models
		return getJdbcTemplate().query(query.toString(), 
								new SimpleStringMapper(), 
								new BeanPropertySqlParameterSource(rc));
	}	
	
	/**
	 * 
	 */
	public List<String> listKeywordsForReportId(ReportCriteria rc) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select keywords ");
		sql.append("from weekly_availability_keyword_critical_states ");
		sql.append("where reportid = :reportId");
		
		// Execute query to get keywords for this report ID
		Map params = Collections.singletonMap("reportId", rc.getReportId());
		return getJdbcTemplate().query(sql.toString(), new SimpleStringMapper(), params);
	}
	
	/**
	 * 
	 */
	public String getAnnotationForReportId(ReportCriteria rc) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select annotation ");
		sql.append("from weekly_availability_keyword_critical_states ");
		sql.append("where reportid = :reportId");
		
		// Execute query to get keywords for this report ID
		Map params = Collections.singletonMap("reportId", rc.getReportId());
		return getJdbcTemplate().queryForObject(sql.toString(), new SimpleStringMapper(), params);
	}
	
	/**
	 * 
	 */
	public int getReportTypeByReportId(int id) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select type ");
		sql.append("from reports ");
		sql.append("where id = :reportId");
		
		// Execute query to get keywords for this report ID
		Map params = Collections.singletonMap("reportId", id);
		return getJdbcTemplate().queryForObject(sql.toString(), new SimpleIntMapper(), params);
	}
	
	/**
	 * 
	 */
	public int getStartOfWeekByReportId(int id) {
		
		StringBuffer sql = new StringBuffer();
		sql.append("select start_of_week ");
		sql.append("from weekly_availability_keyword_critical_states ");
		sql.append("where reportid = :reportId");
		
		// Execute query to get keywords for this report ID
		Map params = Collections.singletonMap("reportId", id);
		return getJdbcTemplate().queryForObject(sql.toString(), new SimpleIntMapper(), params);
	}
}
