package com.altinity.opsview.odw.elements;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.ServiceCheck;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.ServiceCheckKey;
import com.altinity.opsview.odw.reports.ReportCriteria;

public class WeeklyServiceHelper {

	private ReportCriteria rc;
	private ODWReportConfigDAO configDao;
	private ODWReportDAO dao;
	
	/**
	 * 
	 */
	public WeeklyServiceHelper(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao) {
		
		this.rc = rc;
		this.configDao = configDao;
		this.dao = dao;
	}
	
	/**
	 * 
	 */
	public List <ServiceCheckGroup> getServiceChecksByText() throws ReportException {
		
		// Get the keywords for this report
		List <String> keywords = configDao.listKeywordsForReportId(rc);
		
		// Get servicecheck host name and name values for 
		// the specified keywords
		List <ServiceCheckKey> keys = dao.listSCKeysByKeywords(rc, keywords);
		
		// Use this list to sort the final list of all service check groups
		List <ServiceCheckGroup> allGroups = new ArrayList<ServiceCheckGroup>();
		
		// Now get all service checks with host name/name keys within the 
		// reporting period
		for ( ServiceCheckKey key : keys ) {
			
			List <ServiceCheck> serviceChecks = dao.listServiceChecksByKey(rc, key);
			
			// Add service checks to group
			ServiceCheckGroup group = new ServiceCheckGroup();
			group.setServiceChecks(serviceChecks);
			
			// Add group to list of all groups
			// We need this list to sort groups on 
			// text 
			allGroups.add(group);
		}
		
		// Now create unique labels where necessary
		// Do not need this step if using hostname/servicecheck 
		// as labels - they are already unique
		if ( !rc.isUseHostNameServiceCheckLabel() )
			createUniqueLabels(rc, allGroups);

		// Finally return the sorted service check groups
		return sortServiceChecks(allGroups);
	}
	
	/**
	 * 
	 */
	private void createUniqueLabels(ReportCriteria rc, List <ServiceCheckGroup> groups) {
		
		// Check each group has a unique label
		for ( ServiceCheckGroup group : groups ) {
			
			// Update every group that shares this label
			// Keep updating labels until no duplicates exist
			boolean duplicatesExist = true;
			while ( duplicatesExist ) {
				
				String text = group.getText();
				
				// Pessimistic check - this
				// will break loop if no duplicates 
				// are found
				duplicatesExist = false;
				
				// Keep track of whether any duplicated were found
				boolean duplicated = false;
				
				// Check all groups for duplicate labels
				for ( ServiceCheckGroup duplicateGroup : groups ) {
					
					// Don't worry if this is not a duplicate
					// or if it is the same group as the one we 
					// are checking against
					if ( !duplicateGroup.getText().equals(text) ||
							duplicateGroup.getKey().equals(group.getKey()) )
						continue;
					
					// Otherwise, this is a duplicate
					duplicatesExist = true;
					duplicated = true;

					// Update label
					if ( duplicateGroup.getText().equals(text) )
						updateLabel(rc, duplicateGroup);
				}
				
				// Update actual group, but only if its text has not yet
				// been updated and there were duplicates
				if ( duplicated && group.getText().equals(text) )
					updateLabel(rc, group);
			}
		}
	}
	
	/**
	 * 
	 */
	private void updateLabel(ReportCriteria rc, ServiceCheckGroup group) {
		
		// Handle case where servicecheck names are used
		if ( rc.isUseServiceCheckLabel() ) {
			updateUseServiceCheckLabel(group);
		}
		// Handle case where descriptions are used
		else if ( rc.isUseDescriptionLabel() && group.hasNullOrEmptyDescription() ) {
			updateUseServiceCheckLabel(group);
		}
		else
			updateUseDescriptionLabel(group);
	}

	/**
	 * 1. servicecheck name
	 * 2. servicecheck name: hostname
	 */
	private void updateUseServiceCheckLabel(ServiceCheckGroup group) {
		
		String text = group.getText();
		
		// We just want to add the servicecheck name at first
		// This check is required for the case where a servicecheck
		// has no description
		if ( !text.contains(group.getName()) )
			group.setText(group.getName());
		
		// If the text already contains the servicecheck
		// name, suffix with hostname
		else {
			StringBuffer newText = new StringBuffer();
			newText.append(text);
			newText.append(": ");
			newText.append(group.getHostName());
			group.setText(newText.toString());
		}
	}
	
	/**
	 * 1. description
	 * 2. description: hostname
	 * 3. description: hostname (servicecheck name)
	 */
	private void updateUseDescriptionLabel(ServiceCheckGroup group) {
		
		String text = group.getText();
		
		// First suffix description with hostname
		if ( !text.contains(group.getHostName()) ) {
			
			StringBuffer newText = new StringBuffer();
			newText.append(text);
			newText.append(": ");
			newText.append(group.getHostName());
			group.setText(newText.toString());
		}
		
		// Otherwise, we also need to add the servicecheck name
		else {
			
			StringBuffer newText = new StringBuffer();
			newText.append(text);
			newText.append(" (");
			newText.append(group.getName());
			newText.append(")");
			group.setText(newText.toString());
		}
	}
	
	/**
	 * 
	 */
	private List <ServiceCheckGroup> sortServiceChecks(List <ServiceCheckGroup> listToSort) {
		
		// Finally, sort service checks
		Collections.sort(listToSort, new Comparator<ServiceCheckGroup>() {
			
			public int compare(ServiceCheckGroup group0, ServiceCheckGroup group1) {
				
				Comparator<String> comparator = String.CASE_INSENSITIVE_ORDER;
				return comparator.compare(group0.getText(), group1.getText());
			}
		});
		
		return listToSort;
	}
}
